<?php 
/**
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.1
 * 
 * $Id: topic.php 1088 2006-10-28 11:56:59Z jorgenboard $
 * 
 */

//yes, we are here
define("VBOARD", 1);

//==========================
//	Topic page, Handles topic stuff
//==========================

require_once('inc/functions.php');
require_once("inc/lib/pages.php");

class topic {

	/**
	 * Topic
	 *
	 * Decides what should happen in topic.php
	 * @param int post
	 * @return void
	 */
	function topic($user, $db, $config, $cache, $screen)
	{
		//Set variables
		$this->user 	=	$user;
		$this->db   	=	$db;
		$this->config	=	$config;
		$this->cache	=	$cache;
		$this->screen	=	$screen;
		$this->lang 	=	new lang_topic;

		// Choose what to do
		switch($_GET['act'])
		{
			// Find the newest post
			case 'lastpost':
				$this->goToLastPost($_GET['t']);
			break;

			// Delete a post
			case 'delete':
				$this->delete_post($_GET['post']);
			break;

			// Add a post to quote
			case 'add_quote':
				$this->add_quote($_GET['p']);
			break;

			// Subscribe to topic
			case 'subscribe':
				$this->add_topic_subscription($_GET['t']);
			break;

			// Show a topic
			default:
				$this->show_topic($_GET['s']);
			break;
		}
	}

	/**
	 * Go to last post
	 *
	 * Finds the latest post in a topic and sends the user to it
	 * @param int topic
	 * @return void
	 */
	function goToLastPost($topic) {
		
		//Start by finding posts in this topic
		$db_topic	=	$this->cache->query('get_topic_information', array($topic));
		$replies	=	($db_topic[0]->replys + 1); //Get corret numer of posts
		$pages		=	ceil($replies / $this->config->show_maxposts);
		$pages		=	(($pages * $this->config->show_maxposts) - $this->config->show_maxposts);
		$lastPost	=	$db_topic[0]->lpost_pid;		

		// Off we go :)
		header("Location: topic.php?t={$topic}&s={$pages}#{$lastPost}");
	}



	/**
	 * Delete post
	 *
	 * Deletes a single post (or topic if the post is #1 in topic). Checks if the user got sufficent rights to delete the post/topic first.
	 * @param int post
	 * @return void
	 */
	function delete_post($post) {
	  
		// Get information about the post
		$sql		=	$this->db->get_post_information($post);
		$sql		=	$sql->row();
		$ownedby	=	$sql->author_id;
		$forum		=	$sql->forum;

		// Finding which topic and if it's the first post in the topic
		$topic		=	$sql->tid;
		$first_post =	$this->db->first_post_in_topic($topic);
		$first_post	=	$first_post->id;

		$confirmed 	=	$_GET['confirmed'];
		
		// Delete whole topic if post is #1 in topic (let's also ask first)
		if ($first_post == $post)
		{
			if ($confirmed == 'yes')
			{
				// Make sure that the user is allowed to delete topics, and delete it if so
				if ($this->user->g_can_delete_own_topics && ($this->user->id == $ownedby))
				{
					$this->db->delete_topic($topic);
					$num = $this->db->count_posts_in_topic($topic);	//Count number of posts to delete
					$num = $num->total;
				
					$this->db->delete_posts_in_topic($topic, $forum, $num); //Then actually delete the posts

					//get latest topic
					$lasttopic = $this->db->get_last_topic($forum);
				
					//and then update lastpost stats
					$this->db->update_last_topic($forum, $lasttopic->title, $lasttopic->id, $lasttopic->lpost_id, $lasttopic->lpost_uid, $lasttopic->lpost_time, $lasttopic->lpost_name);
					
					$topic_deleted	=	$this->lang->topic_deleted;			
				}
				// User is admin or moderator, can do whatever they want (woha, mod power!)
				elseif ($this->user->g_is_admin || $this->user->g_is_mod )
				{
					$this->db->delete_topic($topic, $forum);		//Unlink the topic first
					$num = $this->db->count_posts_in_topic($topic);	//Count number of posts to delete
					$num = $num->total;
				
					$this->db->delete_posts_in_topic($topic, $forum, $num); //Then actually delete the posts
					
					//get latest topic
					$lasttopic = $this->db->get_last_topic($forum);
				
					//and then update lastpost stats
					$this->db->update_last_topic($forum, $lasttopic->title, $lasttopic->id, $lasttopic->lpost_id, $lasttopic->lpost_uid, $lasttopic->lpost_time, $lasttopic->lpost_name);
					
					$topic_deleted	=	$this->lang->topic_deleted;
				}
				// User do not have the sufficent rights to delete a topic
				else
				{
					$error	=	$this->lang->cant_delete_topic;
				}	
				
			}
			else
			{
				confirm($this->lang->confirm_delete_topic, array($this->lang->yes => "topic.php?act=delete&amp;post={$post}&amp;t={$topic}&amp;confirmed=yes", $this->lang->no => "topic.php?t=$topic"));
			}
			
		}
		
		// Post was not #1 in topic, just delete that single post
		else 
		{
			if ($confirmed == 'yes')
			{	
				// Make sure that the user is allowed to delete posts, and delete it if so
				if ($this->user->g_can_delete_own_posts && ($this->user->id == $ownedby))
				{
					$this->db->delete_post($post, $topic, $forum);
					
					//and then revert lastpost stats
					$lastpost = $this->db->get_last_post($topic);
					$this->db->update_last_post($topic, $forum, $lastpost->author_name, $lastpost->id, $lastpost->author_id, $lastpost->posted, $topic_name);
					
					$post_deleted	=	$this->lang->post_deleted;
				}
				// User is admin or moderator, can do whatever they want
				elseif ($this->user->g_is_admin || $this->user->g_is_mod )
				{
					$this->db->delete_post($post, $topic, $forum);
					
					//and then revert lastpost stats
					$lastpost = $this->db->get_last_post($topic);
					$this->db->update_last_post($topic, $forum, $lastpost->author_name, $lastpost->id, $lastpost->author_id, $lastpost->posted);
					
					$post_deleted	=	$this->lang->post_deleted;
				}
				// User do not have the sufficent rights to delete a post
				else
				{
					$error	=	$this->lang->cant_delete;
				}
			}
			else 
			{
				confirm($this->lang->confirm_delete_post, array($this->lang->yes => "topic.php?act=delete&amp;post={$post}&amp;t={$topic}&amp;confirmed=yes", $this->lang->no => "topic.php?t=$topic"));
			}
		}
		
		// Post was deleted
		if (!$error && $post_deleted)
		{
			notify($post_deleted, "topic.php?t=$topic", $this->lang->go_back_topic);
		}
		// Topic was deleted
		elseif (!$error && $topic_deleted)
		{
			notify($topic_deleted, "forum.php?f=$forum", $this->lang->go_back_forum);
		}
		// Nothing was deleted
		else 
		{
			error_msg($error, 1);
		}
	}


	/**
	 * Add topic subscription
	 *
	 *
	 * @param int topic
	 * @return void
	 */
	function add_topic_subscription($topic)
	{
		global $db, $user;

		if($topic)
		{
			$lang	=	new lang_topic;
			$data	=	$db->get_topic_subscription($user->id, $topic);
	
			if(!$data->id)
			{
				$db->new_topic_subscription($user->id, $topic);
				notify($lang->new_topic_subscription);
			} else {
				notify($lang->existing_topic_subscription);
			}
		}
	}


	/**
	 * Add multiquote
	 *
	 * Adds post to multiquote cookie
	 * @param int post_id
	 * @return void
	 */
	function add_quote($post_id)
	{
		if($post_id && !$this->check_for_quote($post_id))
		{
			// Set a cookie containing post to quote 
			$posts_sofar	=	$_COOKIE["{$this->config->cookie}_multiquote"];
			$posts_sofar	.=	"_$post_id";

			setcookie("{$this->config->cookie}_multiquote", $posts_sofar);
		}
		// Remove a checked posts if we want to.
		else if($this->check_for_quote($post_id))
		{
			$posts_sofar	=	$_COOKIE["{$this->config->cookie}_multiquote"];
			$posts_sofar	=	str_replace("_{$post_id}", "", $posts_sofar);

			setcookie("{$this->config->cookie}_multiquote", $posts_sofar);
		}

		// Off we go :)
		if($_SERVER['HTTP_REFERER'])
		{
			header('Location: ' . $_SERVER['HTTP_REFERER']);
		} else {
			header('Location: topic.php?t=' . $_GET['t']);
		}
	}

	/**
	 * Check for quote
	 *
	 * Checks the multiquote cookie and returns xHTML that "checks" a xHTML checkbox
	 * @param int post_id
	 * @return string
	 */
	function check_for_quote($post_id)
	{
		if(strstr($_COOKIE["{$this->config->cookie}_multiquote"], "_{$post_id}") !== FALSE)
		{
			//returns checked to check of the check box (.p)
			return 'style="font-style: italic" ';
		}
		else
		{
			return '';
		}
	}

	/**
	 * Show topic
	 *
	 * Gets posts from database, checks for user permissions, adds buttons (edit/delete/report/reply/multiquote) and so on
	 * @param int startPost
	 * @return void
	 */
	function show_topic($startPost)
	{
		//	A little check to make sure that we got a topic to show
		if (!is_numeric($_GET['t']))
			error_msg($this->lang->notopic);

		$topic_id = (int)$_GET['t'];

		//	Starting at witch post?
		if(!is_numeric($startPost) || $startPost < 0)
            $startPost  =   0;

		//	Updating the topic read count
		$this->db->update_topic_view_counter($topic_id);

		//	Getting the topic information
		$db_topic	=	$this->cache->query('get_topic_information', array($topic_id));

		//	Found a topic?
		if (!is_array($db_topic))
		{
			error_msg($this->lang->notopic, 1);
		}

		//	Getting post ID`s to select in the next query
		$db_postID	=	$this->db->get_postID_in_topic($topic_id, $startPost, $this->config->show_maxposts);
		$db_postIDrows	=	$db_postID->rows;
		while($row = $db_postID->row())
		{
			$done++;
			$post_ID .= $row->id;
			if($done != $db_postIDrows) {
				$post_ID .=", ";
			}

		}
		$post_ID	.=	" 0";

		//	Updating forum read cookie (yummy!)
		setcookie($config->cookie . "_forumread[" . $_GET['f'] . "]", forumtime(), forumtime() + (86400 * 365), "/");

		//Getting the posts in this topic
		$db_posts	=	$this->db->get_posts_in_topic($post_ID);
		unset($done, $row, $post_ID);

		//	Topic title and topic info in general
		$db_topic   =   $db_topic[0];
		$navigation["index.php?c={$db_topic->catid}"]	=	$db_topic->catname;
		$navigation["forum.php?f={$db_topic->fid}"]		=	$db_topic->fname;
		$navigation["topic.php?t={$topic_id}"]			=	$db_topic->title;

		//==========================
		//	Build up new foo buttons
		//==========================

		// Post-office open?
		if($this->user->g_create_new_posts)
		{
			$html['newtopic']	=	$this->lang->start_newtopic;
			$html['forumID']	=	$db_topic->fid;
			$html['skin_set']	=	$this->user->skin_set;
			$newtopic			=	$this->screen->return_html('button_newtopic', $html);
			unset($html);

			if($db_topic->status == 1) {
				$html['forumID']	=	$db_topic->fid;
				$html['topicID']	=	$db_topic->id;
				$html['reply']		=	$this->lang->topic_reply;
				$html['quickReply'] =	$this->lang->topic_qreply;
				$html['skin_set'] 	=	$this->user->skin_set;
				$reply				=	$this->screen->return_html('buttons_newpost', $html);
			}
		}

		// Poll me?
		if($this->user->g_create_new_polls)
		{
			$html['newpoll']	=	$this->lang->start_newpoll;
			$html['forumID']	=	$db_topic->fid;
			$html['skin_set'] 	=	$this->user->skin_set;
			$newpoll			=	$this->screen->return_html('button_newpoll', $html);
		}

		//	Set up status for this thread, to use in the icon for the topic
		switch($db_topic->status)
		{
			case "2":
				$status = "validating";
			break;

			case "0":
				$status = "closed";
			break;

			default:
				$status = "open";
		}

		//Title of the topic please!
		$topic_title	=	$db_topic->title;

		//Construct navigation links pages($current, $last, $href, $multiplier)
		$navdata	=	$this->lang->pages;

		//Get correct page number
		$db_topic->replys++;

		$pages		=	ceil($db_topic->replys / $this->config->show_maxposts);
		$thisPage	=	ceil($startPost / $this->config->show_maxposts);
		$thisPage++;
		$navdata	.=	pages($thisPage, $pages, "topic.php?t={$topic_id}&amp;s=", $this->config->show_maxposts);

		//Get the users skin set so we get the correct images
		$skin_set = $this->user->skin_set;

		// Looping through all posts we are to show on this page
		while($db_post	=	$db_posts->row())
		{
			// Get template and change the usual tags first (witch always are in a "system")
			$post_date					=	"{$this->lang->topic_pos} " . date($this->config->timestamp1, forumtime($db_post->posted));
			$post_id					=	$db_post->id;
			$post_post					=	$db_post->post;
			$this->user_name			=	$db_post->author_name;
			$this->user_id				=	$db_post->author_id;
			$this->user_title			=	$db_post->title;
			$this->user_registred_title	=	$this->lang->top_m_joined;
			$this->user_registred		=	date($this->config->timestamp2, forumtime($db_post->registred));
			$this->user_location		=	$db_post->location;
			$this->user_location_title	=	$this->lang->top_m_loc;
			$this->user_posts			=	$db_post->posts;
			$this->user_posts_title		=	$this->lang->top_m_posts;

			// Buttons
			$button_edit		=	'<a href="post.php?mode=02&amp;p=' . $db_post->id . '"><img src="img/' . $skin_set . '/accessories-text-editor.png" /> ' . $this->lang->top_edit . '</a>';
			$button_delete		=	'<a href="topic.php?act=delete&amp;post=' . $db_post->id . '&amp;t=' . $db_topic->id . '"><img src="img/' . $skin_set . '/edit-delete.png" /> ' . $this->lang->top_del . '</a>';
			$button_report		=	'<a href="report.php?p=' . $db_post->id . '"><img src="img/' . $skin_set . '/dialog-warning-small.png" /> ' . $this->lang->top_rep . '</a>';
			$button_reply		=	'<a href="post.php?mode=03&amp;t=' . $topic_id . '&quote=' . $db_post->id . '"><img src="img/' . $skin_set . '/document-new.png" /> ' . $this->lang->top_quo . '</a>';
			$button_multiquote	=	'<a href="topic.php?act=add_quote&amp;p=' . $db_post->id . '&amp;t=' . $_GET['t'] . '" id="mq_' . $db_post->id . '" onclick="add_multiquote(' . $db_post->id . ', \'' . $this->config->cookie . '_multiquote\'); edit_multiquote_style(' . $db_post->id . ', \'italic\'); return false;" ' . $this->check_for_quote($db_post->id) . '><img src="img/' . $skin_set . '/format-quote-multi.png" /> ' . $this->lang->top_mquo . '</a>';

			// Show signature?
			if($db_post->use_sig && $db_post->signature)
			{
				$this->user_signature	=	'<br /><br />--------------------------------------<br />' . $db_post->signature;
			}
			else
			{
				$this->user_signature	=	'';
			}

			// Show avatar from post author?
			if ($db_post->avatar)
			{
				$this->user_avatar	=	'<img src="' . $db_post->avatar . '" alt="Avatar for user ' . $this->user_name . '" />';
			}
			else 
			{
				$this->user_avatar = '';
			}

			// Can this user post here?
			if ($this->user->g_create_new_posts)
			{
				// Add buttons
				$post_quote		=	$button_reply;
				$post_mquote	=	$button_multiquote;
				$post_report	=	$button_report;

				// MODERATOR POWER!
				if($this->user->g_is_mod == "1")
				{
					
					$post_report	=	$button_report;
					
					// Mark post red if deleted
					/*if ($db_post->deleted == '1')
					{
						echo "THIS POST IS DELETED";
					}*/

					// Give moderator access for posts
					if ($this->user->id != $db_post->author_id)
					{
						$post_delete	=	$button_delete;
						$post_edit		=	$button_edit;
					}

					// Can a moderator touch own posts?
					if ($this->user->id == $db_post->author_id)
					{
						// Show delete post button on own posts
						if($this->user->g_can_delete_own_posts)
						{
							$post_delete	=	$button_delete;
						}

						// POST editing button for own posts
						if ($this->user->g_can_edit_own_posts)
						{
							$post_edit	=	$button_edit;
						}
					}
				}

				//	Normal user acess
				if (!$this->user->g_is_mod)
				{
					
					$post_report	=	$button_report;
					
					//	This is the user`s post?
					if ($this->user->id == $db_post->author_id)
					{
						//	Can he/her delete it?
						if ($this->user->g_can_delete_own_posts)
						{
							$post_delete	=	$button_delete;
						}

						//	Author may edit it?
						if ($this->user->g_can_edit_own_posts)
						{
							$post_edit	=	$button_edit;
						}
					}
				}
			}

			//	is for all guests...
			else
			{
				//	Can guests post?
				if ($this->user->g_create_new_posts)
				{
					$post_quote		=	$button_quote;
					$post_mquote	=	$button_multiquote;
					$post_report	=	$button_report;
				}
			}

			//	Knock template stuffie
			$post_html	=	array(	
				"post_date" => $post_date,
				"post_id" => $post_id,
				"post_post" => $post_post,
				"post_quote" => $post_quote,
				"post_mquote" => $post_mquote,
				"post_report" => $post_report,
				"post_delete" => $post_delete,
				"post_edit" => $post_edit,
				"user_name" => $this->user_name,
				"user_id" => $this->user_id,
				"user_title" => $this->user_title,
				"user_registred" => $this->user_registred,
				"user_registred_title" => $this->user_registred_title,
				"user_location" => $this->user_location,
				"user_location_title" => $this->user_location_title,
				"user_posts" => $this->user_posts,
				"user_posts_title" => $this->user_posts_title,
				"user_avatar" => $this->user_avatar,
				"post_signature" => $this->user_signature
			);

			$posts	.=	$this->screen->return_html('showtopic_post', $post_html);
		}

		//=============================
		// Quick reply
		//=============================

		$lang_post		= new lang_post;

		$title			=	$lang_post->new_reply;
		$submit_button	=	$lang_post->submit_newpost;
		$preview_button	=	$lang_post->submit_preview;

		// Get forum ID based on topic ID
		$forumID	=	$this->db->get_topic_information($topic_id);
		$forumID	=	$forumID->row();
		$forumID	=	$forumID->fid;

		// Get some information about the category we are in
		$tmp	=	$this->db->get_forum_info($forumID);
		$category = $tmp->row();	
		$category = $category->cat;

		// Some more templating
		$html	=	array(
			"alternaltives"		=>	$lang_post->alternaltives,
			"submit_button"		=>	$submit_button,
			"preview_button"	=>	$preview_button,
			"category"			=>	$category,
			"do_what"			=>	'new_reply',
			"forum_id"			=>	$forumID,
			"sid"				=>	$this->user->session,
			"user_id"			=>	$this->user->id,
			"post_id"			=>	$postID,
			"topic_id"			=>	$topic_id,
			"show_signature"	=>	$lang_post->show_signature,
			"show_smileys"		=>	$lang_post->show_smilies
		);

		$quickreply = $this->screen->return_html('post_quick_reply', $html);

		// Add posts data to template
		$header_html = array(
			"topic_id"				=>	$topic_id,
			"topic_title" 			=>	$topic_title,
			"topic_status" 			=>	$status,
			"skin_set"				=>	$this->user->skin_set,
			"newtopic"				=>	$newtopic,
			"newpoll"				=>	$newpoll,
			"reply"					=>	$reply,
			"forum_id"				=>	$forumID,
			"top"					=>	$this->lang->top_top,
			"navigation" 			=>	$navdata,
			"posts" 				=>	$posts,
			'title-quick-reply'		=>	$this->lang->topic_qreply,
			'quick-reply-area'		=>	$quickreply,
			'topic_subscription'	=>	$lang_post->topic_subscription,
		);

		$this->screen->add_html('showtopic_body', $header_html);

		// Finish off by returning page to user
		$this->screen->add_wrapper($db_topic->title, $navigation);
		$this->screen->print_page();
	}
}

//START :)
$topic = new topic($user, $db, $config, $cache, $screen);


?>